@@ -6,7 +6,7 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: List.scala 8958 2006-10-13 19:06:41Z michelou $
+// $Id: List.scala 9150 2006-11-05 02:53:14Z emir $
 
 
 package scala
@@ -371,7 +371,7 @@ object List {
  *  @author  Martin Odersky and others
  *  @version 1.0, 16/07/2003
  */
-sealed abstract class List[+a] extends Seq[a] with CaseClass {
+sealed abstract class List[+a] extends Seq[a] {
 
   /** Returns true if the list does not contain any elements.
    *  @return true, iff the list is empty.
@@ -380,13 +380,13 @@ sealed abstract class List[+a] extends Seq[a] with CaseClass {
 
   /** Returns this first element of the list.
    *  @return the first element of this list.
-   *  @throws <code>java.lang.RuntimeException</code> if the list is empty.
+   *  @throws <code>scala.compat.Platform.NoSuchElementException</code> if the list is empty.
    */
   def head: a
 
   /** Returns this list without its first element.
    *  @return this list without its first element.
-   *  @throws <code>java.lang.RuntimeException</code> if the list is empty.
+   *  @throws <code>scala.compat.Platform.NoSuchElementException</code> if the list is empty.
    */
   def tail: List[a]
 
@@ -472,7 +472,7 @@ sealed abstract class List[+a] extends Seq[a] with CaseClass {
     def hasNext: Boolean = !these.isEmpty
     def next: a =
       if (!hasNext)
-        throw new java.util.NoSuchElementException("next on empty Iterator")
+        throw new NoSuchElementException("next on empty Iterator")
       else {
         val result = these.head; these = these.tail; result
       }
@@ -488,9 +488,10 @@ sealed abstract class List[+a] extends Seq[a] with CaseClass {
   /** Returns the list without its last element.
    *
    *  @return the list without its last element.
-   *  @throws <code>java.lang.RuntimeException</code> if the list is empty.
+   *  @throws <code>scala.compat.Platform.UnsupportedOperationException</code>
+   *  if the list is empty.
    */
-  def init: List[a] = 
+  def init: List[a] =
     if (isEmpty) throw new UnsupportedOperationException("Nil.init")
     else {
       val b = new ListBuffer[a]
@@ -507,7 +508,8 @@ sealed abstract class List[+a] extends Seq[a] with CaseClass {
   /** Returns the last element of this list.
    *
    *  @return the last element of the list.
-   *  @throws <code>java.lang.RuntimeException</code> if the list is empty.
+   *  @throws <code>scala.compat.Platform.UnsupportedOperationException</code>
+   *  if the list is empty.
    */
   def last: a =
     if (isEmpty) throw new UnsupportedOperationException("Nil.last")
@@ -544,7 +546,6 @@ sealed abstract class List[+a] extends Seq[a] with CaseClass {
    *
    *  @param n the number of elements to take
    *  @return the suffix of length <code>n</code> of the list
-   *  @throws <code>java.lang.RuntimeException</code> if the list is too short.
    */
   def takeRight(n: Int): List[a] = {
     def loop(lead: List[a], lag: List[a]): List[a] = lead match {
@@ -558,7 +559,6 @@ sealed abstract class List[+a] extends Seq[a] with CaseClass {
    *
    *  @param n the number of elements to take
    *  @return the suffix of length <code>n</code> of the list
-   *  @throws <code>java.lang.RuntimeException</code> if the list is too short.
    */
   def dropRight(n: Int): List[a] = {
     def loop(lead: List[a], lag: List[a]): List[a] = lead match {
@@ -1079,7 +1079,8 @@ sealed abstract class List[+a] extends Seq[a] with CaseClass {
     b.toList
   }
 
-  override protected def stringPrefix: String = "List"
+  protected override def stringPrefix: String = "List"
+
 }
 
 /** The empty list.
@@ -1090,16 +1091,10 @@ sealed abstract class List[+a] extends Seq[a] with CaseClass {
 [SerialVersionUID(0 - 8256821097970055419L)]
 case object Nil extends List[Nothing] {
   override def isEmpty = true
-  /**
-   *  @throws java.util.NoSuchElementException
-   */
   def head: Nothing =
-    throw new java.util.NoSuchElementException("head of empty list")
-  /**
-   *  @throws java.util.NoSuchElementException
-   */
+    throw new NoSuchElementException("head of empty list")
   def tail: List[Nothing] =
-    throw new java.util.NoSuchElementException("tail of empty list")
+    throw new NoSuchElementException("tail of empty list")
 }
 
 /** A non empty list characterized by a head and a tail.
