@@ -2,7 +2,7 @@
  * Copyright 2005-2006 LAMP/EPFL
  * @author  Sean McDirmid
  */
-// $Id: DocGenerator.scala 9022 2006-10-23 17:49:33Z michelou $
+// $Id: DocGenerator.scala 9151 2006-11-05 19:11:08Z mihaylov $
 
 package scala.tools.nsc.doc
 
@@ -10,6 +10,7 @@ import java.io.{File, FileOutputStream, FileWriter}
 import java.util.StringTokenizer
 import java.util.regex.Pattern
 
+import compat.Platform.{EOL => LINE_SEPARATOR}
 import scala.collection.immutable._
 import scala.collection.mutable.ListBuffer
 import scala.tools.nsc._
@@ -794,7 +795,7 @@ abstract class DocGenerator extends Models {
       def sym = definitions.getClass("scala.runtime.BoxedLong")
     }
     new PrimitiveContentFrame {
-      def sym = definitions.BoxedNumberClass
+      def sym = definitions.getClass("scala.runtime.BoxedNumber")
     }
     val rsrcdir = "scala/tools/nsc/doc/".replace('/', File.separatorChar)
     for (val base <- List("style.css", "script.js")) {
@@ -938,8 +939,6 @@ abstract class DocGenerator extends Models {
 
   val root = <b></b>
 
-  private val LINE_SEPARATOR = System.getProperty("line.separator", "\n")
-
   private val NAME_SUFFIX_OBJECT  = "$object"
   private val NAME_SUFFIX_PACKAGE = "$package"
   private val FILE_EXTENSION_HTML = ".html"
