@@ -2,7 +2,7 @@
  * Copyright 2005-2006 LAMP/EPFL
  * @author Martin Odersky
  */
-// $Id: Erasure.scala 9000 2006-10-20 13:13:38Z odersky $
+// $Id: Erasure.scala 9151 2006-11-05 19:11:08Z mihaylov $
 
 package scala.tools.nsc.transform
 
@@ -141,6 +141,16 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
    }
   }
 
+  private def evalOnce(expr: Tree, owner: Symbol, unit: CompilationUnit)
+                      (within: (() => Tree) => Tree): Tree =
+    if (treeInfo.isPureExpr(expr)) {
+      within(() => expr);
+    } else {
+      val temp = owner.newValue(expr.pos, unit.fresh.newName())
+      .setFlag(SYNTHETIC).setInfo(expr.tpe);
+      Block(List(ValDef(temp, expr)), within(() => Ident(temp) setType expr.tpe))
+    }
+
 // -------- boxing/unboxing --------------------------------------------------------
 
   override def newTyper(context: Context) = new Eraser(context)
@@ -148,15 +158,6 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
   /** The modifier typer which retypes with erased types. */
   class Eraser(context: Context) extends Typer(context) {
 
-    private def evalOnce(expr: Tree, within: (() => Tree) => Tree): Tree =
-      if (treeInfo.isPureExpr(expr)) {
-        within(() => expr);
-      } else {
-        val temp = context.owner.newValue(expr.pos, context.unit.fresh.newName())
-          .setFlag(SYNTHETIC).setInfo(expr.tpe);
-        Block(List(ValDef(temp, expr)), within(() => Ident(temp) setType expr.tpe))
-      }
-
     /** Box `tree' of unboxed type */
     private def box(tree: Tree): Tree =
       typed {
@@ -171,8 +172,8 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
                              else BoxedObjectArrayClass;
             Apply(Select(New(TypeTree(boxedClass.tpe)), nme.CONSTRUCTOR), List(tree))
           } else {
-            val boxedModule = boxedClass(tree.tpe.symbol).linkedModuleOfClass;
-            Apply(Select(gen.mkAttributedRef(boxedModule), nme.box), List(tree))
+            Apply(gen.mkAttributedRef(boxMethod(tree.tpe.symbol)), List(tree)).
+              setPos(tree.pos) setType ObjectClass.tpe
           }
         }
       }
@@ -185,14 +186,6 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
         }
       }
 
-    /** The method-name xxxValue, where Xxx is a numeric value class name */
-    def unboxOp(tp: Type): Name = {
-      val clazzName = tp.symbol.name.toString()
-      newTermName(
-        String.valueOf((clazzName.charAt(0) + ('a' - 'A')).asInstanceOf[char]) +
-        clazzName.substring(1) + "Value")
-    }
-
     /** Unbox `tree' of boxed type to expected type `pt' */
     private def unbox(tree: Tree, pt: Type): Tree =
       typed {
@@ -200,28 +193,14 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
           if (pt.symbol == UnitClass) {
             if (treeInfo.isPureExpr(tree)) Literal(())
             else Block(List(tree), Literal(()))
-          } else {
-            if (pt.symbol == BooleanClass) {
-              val tree1 = adaptToType(tree, boxedClass(BooleanClass).tpe)
-              gen.mkRuntimeCall(nme.booleanValue, List(tree1))
-            } else if (pt.symbol == ArrayClass) {
-              val tree1 = adaptToType(tree, BoxedArrayClass.tpe)
-/*
-              val elemClass = pt.typeArgs.head.symbol;
-              val elemTag = 
-                if (isValueClass(elemClass)) 
-                  gen.mkRuntimeCall(newTermName(elemClass.name.toString() + "Tag"), List())
-                else 
-                  Literal(signature(pt.typeArgs.head));
-*/
-              //System.out.println("unboxing " + tree + ":" + tree.tpe + " to " + pt);//DEBUG
-              //gen.mkRuntimeCall(nme.arrayValue, List(tree1, elemTag))
-              gen.mkRuntimeCall(nme.arrayValue, List(tree1, Literal(pt.typeArgs.head)))
-            } else {
-              assert(isNumericValueClass(pt.symbol));
-              val tree1 = adaptToType(tree, BoxedNumberClass.tpe);
-              gen.mkRuntimeCall(unboxOp(pt), List(tree1))
-            }
+          }
+          else if (pt.symbol == ArrayClass) {
+            val tree1 = adaptToType(tree, BoxedArrayClass.tpe)
+            gen.mkRuntimeCall(nme.arrayValue, List(tree1, Literal(pt.typeArgs.head)))
+          }
+          else {
+            Apply(gen.mkAttributedRef(unboxMethod(pt.symbol)), List(tree)).
+              setPos(tree.pos) setType pt
           }
         }
       }
@@ -237,7 +216,7 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
         if (pt.symbol == ArrayClass)
           typed {
             atPos(tree.pos) {
-              evalOnce(tree, x =>
+              evalOnce(tree, context.owner, context.unit) { x =>
                 gen.mkAttributedCast(
                   If(
                     Apply(
@@ -247,13 +226,14 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
                       List()),
                     unbox(gen.mkAttributedCast(x(), BoxedArrayClass.tpe), pt),
                     x()),
-                  pt))
+                  pt)
+              }
             }
           }
         else if (pt.symbol isNonBottomSubClass BoxedArrayClass)
           typed {
             atPos(tree.pos) {
-              evalOnce(tree, x =>
+              evalOnce(tree, context.owner, context.unit) { x =>
                 gen.mkAttributedCast(
                   If(
                     Apply(
@@ -263,13 +243,14 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
                       List()),
                     x(),
                     boxArray(x())),
-                  pt))
+                  pt)
+              }
             }
           }
         else if ((SeqClass isNonBottomSubClass pt.symbol) && pt.symbol != ObjectClass)
           typed {
             atPos(tree.pos) {
-              evalOnce(tree, x =>
+              evalOnce(tree, context.owner, context.unit) { x =>
                 gen.mkAttributedCast(
                   If(
                     Apply(
@@ -279,7 +260,8 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
                       List()),
                     x(),
                     boxArray(x())),
-                  pt))
+                  pt)
+              }
             }
           }
         else gen.mkAttributedCast(tree, pt)
@@ -373,7 +355,7 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
             if (isUnboxedClass(tree.symbol.owner) && !isUnboxedClass(qual1.tpe.symbol))
               tree.symbol = NoSymbol
             else if (qual1.tpe.isInstanceOf[MethodType] && qual1.tpe.paramTypes.isEmpty) {
-              assert(qual1.symbol.isStable);
+              assert(qual1.symbol.isStable, qual1.symbol);
               qual1 = Apply(qual1, List()) setPos qual1.pos setType qual1.tpe.resultType;
             } else if (!(qual1.isInstanceOf[Super] || (qual1.tpe.symbol isSubClass tree.symbol.owner)))
               qual1 = cast(qual1, tree.symbol.owner.tpe);
@@ -395,7 +377,7 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
       } catch {
         case ex: Throwable =>
           //if (settings.debug.value) 
-            System.out.println("exception when typing " + tree);
+            Console.println("exception when typing " + tree);
           throw ex
       }
       def adaptCase(cdef: CaseDef): CaseDef = {
@@ -637,13 +619,28 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
               // todo: also handle the case where the singleton type is buried in a compound
 	    else if (fn.symbol == Any_isInstanceOf || fn.symbol == Any_isInstanceOfErased)
               fn match {
-                case TypeApply(Select(qual, _), List(targ)) =>
+                case TypeApply(sel @ Select(qual, name), List(targ)) =>
                   targ.tpe match {
                     case SingleType(pre, sym) =>
                       val cmpOp = if (targ.tpe <:< AnyValClass.tpe) Any_equals else Object_eq
                       atPos(tree.pos) {
                         Apply(Select(qual, cmpOp), List(gen.mkAttributedQualifier(targ.tpe)))
                       }
+                    case RefinedType(parents, decls) if (parents.length >= 2) =>
+                      evalOnce(qual, currentOwner, unit) { 
+                        q =>
+                        def mkIsInstanceOf(tp: Type) =
+                          copy.Apply(tree, 
+                            copy.TypeApply(fn, 
+                              copy.Select(sel, q(), name), 
+                              List(TypeTree(tp))),
+                            List())
+                        def mkAnd(tree1: Tree, tree2: Tree) = 
+                          atPos(tree1.pos) {
+                            Apply(Select(tree1, Boolean_and), List(tree2))
+                          }
+                        parents map mkIsInstanceOf reduceRight mkAnd
+                      }
                     case _ =>
                       tree
                   }
@@ -653,7 +650,7 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
 
           case Template(parents, body) =>
             assert(!currentOwner.isImplClass)
-            //System.out.println("checking no dble defs " + tree)//DEBUG
+            //Console.println("checking no dble defs " + tree)//DEBUG
             checkNoDoubleDefs(tree.symbol.owner)
             copy.Template(tree, parents, addBridges(body, currentOwner))
           case _ =>
